Package: motifTestR
Title: Perform key tests for binding motifs in sequence data
Version: 1.7.0
Authors@R: person("Stevie", "Pederson", 
  email = "stephen.pederson.au@gmail.com", 
  role = c("aut", "cre"),
  comment = c(ORCID = "0000-0001-8197-3303")
  )
Description: Taking a set of sequence motifs as PWMs, test a set of sequences
  for over-representation of these motifs, as well as any positional features
  within the set of motifs. Enrichment analysis can be undertaken using multiple
  statistical approaches. The package also contains core functions to prepare 
  data for analysis, and to visualise results.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/smped/motifTestR
BugReports: https://github.com/smped/motifTestR/issues
Depends: Biostrings, GenomicRanges, ggplot2 (>= 4.0.0), R (>= 4.5.0),
Imports: Seqinfo, graphics, harmonicmeanp, IRanges, matrixStats,
        methods, parallel, patchwork, rlang, S4Vectors, stats,
        universalmotif,
Suggests: AnnotationHub, BiocStyle, BSgenome.Hsapiens.UCSC.hg19,
        extraChIPs (>= 1.13.3), ggdendro, knitr, MASS, MotifDb,
        rmarkdown, rtracklayer, SimpleUpset, testthat (>= 3.0.0), VGAM
biocViews: MotifAnnotation, ChIPSeq, ChipOnChip, SequenceMatching,
        Software
LazyData: false
RoxygenNote: 7.3.3
Roxygen: list(markdown = TRUE)
Config/testthat/edition: 3
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/motifTestR
git_branch: devel
git_last_commit: 26a47a7
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: no
Packaged: 2025-10-31 23:00:09 UTC; biocbuild
Author: Stevie Pederson [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-8197-3303>)
Maintainer: Stevie Pederson <stephen.pederson.au@gmail.com>
