# Generated by roxygen2: do not edit by hand

export(clusterMotifs)
export(countClusterMatches)
export(countPwmMatches)
export(getClusterMatches)
export(getPwmMatches)
export(makeRMRanges)
export(plotMatchPos)
export(simMultiMotifs)
export(simSeq)
export(testClusterEnrich)
export(testClusterPos)
export(testMotifEnrich)
export(testMotifPos)
exportMethods(makeRMRanges)
import(Biostrings)
import(GenomicRanges)
import(ggplot2)
importClassesFrom(universalmotif,universalmotif)
importFrom(IRanges,DataFrameList)
importFrom(IRanges,IRanges)
importFrom(IRanges,IRangesList)
importFrom(IRanges,IntegerList)
importFrom(IRanges,Views)
importFrom(IRanges,findOverlaps)
importFrom(IRanges,overlapsAny)
importFrom(IRanges,width)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,DataFrame)
importFrom(S4Vectors,endoapply)
importFrom(S4Vectors,mcols)
importFrom(S4Vectors,queryHits)
importFrom(S4Vectors,splitAsList)
importFrom(S4Vectors,subjectHits)
importFrom(Seqinfo,seqinfo)
importFrom(graphics,abline)
importFrom(harmonicmeanp,p.hmp)
importFrom(matrixStats,colSds)
importFrom(matrixStats,rowSums2)
importFrom(methods,as)
importFrom(methods,is)
importFrom(methods,slot)
importFrom(parallel,mclapply)
importFrom(patchwork,plot_layout)
importFrom(rlang,"!!")
importFrom(rlang,sym)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.formula)
importFrom(stats,binom.test)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,glm)
importFrom(stats,hclust)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,phyper)
importFrom(stats,poisson)
importFrom(stats,poisson.test)
importFrom(stats,quasipoisson)
importFrom(stats,setNames)
importFrom(stats,xtabs)
importFrom(universalmotif,compare_motifs)
importFrom(universalmotif,convert_type)
importFrom(universalmotif,create_motif)
