% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ar_er_peaks}
\alias{ar_er_peaks}
\title{A set of peaks with AR and ER detected}
\format{
An object of class \code{GRanges} of length 849.
}
\source{
\url{https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE123767}
}
\usage{
data("ar_er_peaks")
}
\description{
A set of ChIP-Seq peaks where AR and ER were both detected
}
\details{
The subset of peaks found on chr1 which contained signal from at least
two of AR, ER and H3K27ac, taken from GSE123767.
Peaks were resized to a uniform width of 400bp after downloading

Generation of these ranges is documented in
\code{system.file("scripts/ar_er_peaks.R", package = "motifTestR")}
}
\examples{
data("ar_er_peaks")
ar_er_peaks
}
\keyword{datasets}
