% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{zr75_enh}
\alias{zr75_enh}
\title{Candidate Enhancer Regions from ZR-75-1 Cells}
\format{
An object of class \code{GRanges} of length 5237.
}
\source{
\url{http://www.enhanceratlas.org/index.php}
}
\usage{
data("zr75_enh")
}
\description{
The chr1 subset of candidate enhancers for ZR-75-1 cells
}
\details{
These enhancers are the chr1 subset of enhancer regions for ZR-75-1 cells as
identified by EnhancerAtlas 2.0

#' Generation of these ranges is documented in
\code{system.file("scripts/zr75_enh.R", package = "motifTestR")}
}
\examples{
data("zr75_enh")
zr75_enh
}
\keyword{datasets}
