% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ar_er_seq}
\alias{ar_er_seq}
\title{Sequences from peaks with AR and ER detected}
\format{
An object of class \code{DNAStringSet} of length 849.
}
\usage{
data("ar_er_seq")
}
\description{
The genomic sequences obtained from the ar_er_peaks
}
\details{
These sequences represent the sequences obtained from
BSgenome.Hsapiens.UCSC.hg19 for thw peaks supplied as \code{ar_er_peaks}

Generation of these sequences is documented in
\code{system.file("scripts/ar_er_peaks.R", package = "motifTestR")}
}
\examples{
data("ar_er_seq")
ar_er_seq
}
\keyword{datasets}
