% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{hg19_mask}
\alias{hg19_mask}
\title{Regions from hg19 with high N content}
\format{
An object of class \code{GRanges} of length 345.
}
\source{
The package AnnotationHub and
\url{https://hgdownload.cse.ucsc.edu/goldenpath/hg19/bigZips/hg19.fa.masked.gz}
}
\usage{
data("hg19_mask")
}
\description{
A GRanges object with regions annotated as telomeres or centromeres
}
\details{
The regions defined as centromeres or telomeres in hg19, taken from
AnnotationHub objects "AH107360" and "AH107361". These were combined with
regions containing Ns from the UCSC 2bit file, and regions with Ns in the
BSgenome.Hsapiens.UCSC.hg19 were retained.

Generation of these ranges is documented in
\code{system.file("scripts/hg19_mask.R", package = "motifTestR")}
}
\examples{
data("hg19_mask")
hg19_mask
}
\keyword{datasets}
