% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{ex_pfm}
\alias{ex_pfm}
\title{Example Position Frequency Matrices}
\format{
An object of class \code{list} of length 5.
}
\usage{
data("ex_pfm")
}
\description{
Example Position Frequency Matrices
}
\details{
This object contains 5 PFMs taken from HOCOMOCOv11-coreA for examples and testing

Generation of this motif list is documented in
\code{system.file("scripts/ex_pfm.R", package = "motifTestR")}
}
\examples{
data("ex_pfm")
ex_pfm$ESR1

}
\keyword{datasets}
